@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2017 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -59,7 +59,7 @@ Elements::Elements(std::string epdl97Directory, std::string bindingEnergiesFileN
     // this is to simplify an initialization equivalent to that of PyMca:
     // - use EPDL97 to calculate partial photoelectric cross sections
     // - use a different binding energies file
-    // - use different mass atenuation coefficients (i.e. XCOM)
+    // - use different mass attenuation coefficients (i.e. XCOM)
     this->initialize(epdl97Directory, bindingEnergiesFileName);
     if (crossSectionsFile.size())
     {
@@ -188,7 +188,14 @@ void Elements::initialize(std::string epdl97Directory, std::string bindingEnergi
         symbol = defaultElementsInfo[i].symbol;
         atomicNumber = defaultElementsInfo[i].z;
         this->elementList[i] = Element(symbol, atomicNumber);
+        this->elementList[i].setColumn(defaultElementsInfo[i].column);
+        this->elementList[i].setRow(defaultElementsInfo[i].row);
+        this->elementList[i].setLongName(defaultElementsInfo[i].longName);
         this->elementList[i].setAtomicMass(defaultElementsInfo[i].atomicMass);
+        if (defaultElementsInfo[i].density > 0.0000000001)
+        {
+            this->elementList[i].setDensity((defaultElementsInfo[i].density / 1000.));
+        }
         this->elementList[i].setBindingEnergies(epdl97.getBindingEnergies(atomicNumber));
         massAttenuationCoefficients = epdl97.getMassAttenuationCoefficients(atomicNumber);
         this->elementList[i].setMassAttenuationCoefficients(massAttenuationCoefficients["energy"],\
@@ -810,7 +817,7 @@ void Elements::setMassAttenuationCoefficients(const std::string & name,
             shell = shellList[shellIndex];
             if (extractedEdgeEnergies.find(shell) != extractedEdgeEnergies.end())
             {
-                if ((i - j - 1) == extractedEdgeEnergies[shell].second)
+                if ((i - j - 1) == static_cast<unsigned int>(extractedEdgeEnergies[shell].second))
                 {
                     // We are just above an edge according to the input mass attenuation coefficients
                     // We have to check if that energy is compatible with the binding energy.
@@ -1550,6 +1557,7 @@ std::map<std::string, double> Elements::parseFormula(const std::string & formula
     }
 
     // look for parenthesis
+    parsingKey = false;
     spacesPresent = false;
     for(i=0; i < formula.size(); i++)
     {
@@ -1812,12 +1820,41 @@ std::vector<std::pair<std::string, double> > Elements::getPeakFamilies( \
     return result;
 };
 
+const double & Elements::getAtomicMass(const std::string & element) const
+{
+    return getElement(element).getAtomicMass();
+}
+
+const int & Elements::getAtomicNumber(const std::string & element) const
+{
+    return getElement(element).getAtomicNumber();
+}
 
 const std::map<std::string, double> & Elements::getBindingEnergies(const std::string & element) const
 {
     return getElement(element).getBindingEnergies();
 }
 
+double Elements::getDensity(const std::string & element) const
+{
+    return getElement(element).getDensity();
+}
+
+std::string Elements::getLongName(const std::string & element) const
+{
+    return getElement(element).getLongName();
+}
+
+int Elements::getRow(const std::string & element) const
+{
+    return getElement(element).getRow();
+}
+
+int Elements::getColumn(const std::string & element) const
+{
+    return getElement(element).getColumn();
+}
+
 std::vector<std::pair<std::string, double> > Elements::getPeakFamilies( \
                                             const std::string & name, const double & energy) const
 {
@@ -1918,14 +1955,6 @@ std::map<std::string,std::map<std::string, double> > Elements::getEscape( \
             // std::cout << "USING CACHE" <<  energy << std::endl;
             return it->second;
         }
-        else
-        {
-            // std::cout << "ENERGY NOT THERE" << energy << std::endl;
-        }
-    }
-    else
-    {
-        // std::cout << "NOT COMPATIBLE" << std::endl;
     }
 
     if (alphaIn == 90.)
@@ -1942,7 +1971,16 @@ std::map<std::string,std::map<std::string, double> > Elements::getEscape( \
     result.clear();
 
     if (thickness > 0.0)
+    {
         intrinsicEfficiency = (1.0 - std::exp(-muIncident * thickness / sinAlphaIn));
+    }
+    else
+    {
+        // this is just to get rid of a not very clever non-initialized warning because
+        // if thickness is not bigger than 0.0 intrinsicEfficiency will not be used in
+        // any case.
+        intrinsicEfficiency = 1.0;
+    }
     for (c_it = composition.begin(); c_it != composition.end(); c_it++)
     {
         element = c_it->first;
@@ -1951,15 +1989,15 @@ std::map<std::string,std::map<std::string, double> > Elements::getEscape( \
         std::map<std::string, std::map<std::string, double> >::const_iterator it;
         std::map<std::string, double>::const_iterator mapIt;
         // factor uses mu["shell photoelectric"] / mu["total photoelectric"]
-        double factor;
+        // double factor;
         // rate uses mu["shell photoelectric"]
         // It is the product factor * mu["total photoelectric"]
         double rate;
         double fluorescentEnergy;
         for ( it = excitationFactors.begin(); it != excitationFactors.end(); ++it)
         {
-            mapIt = it->second.find("factor");
-            factor = mapIt->second;
+            // mapIt = it->second.find("factor");
+            // factor = mapIt->second;
             mapIt = it->second.find("rate");
             rate = mapIt->second;
             mapIt = it->second.find("energy");
