@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2016 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -120,6 +120,16 @@ public:
     std::vector<std::string> getElementNames();
 
 
+    /*!
+    Convenience method to simplify access to element properties from binding (ex. python)
+
+    Given an element and an excitation energy (in keV), return a map where the key is the line name and the content
+    the energy. The method getPeakFamilies is more complete.
+    */
+    std::map<std::string, double> getEmittedXRayLines(const std::string & elementName, \
+                                                      const double & energy= 1000.) const;
+
+
     /*!
     Convenience method to simplify access to element properties from binding (ex. python)
     */
@@ -365,11 +375,43 @@ public:
     std::vector<std::pair<std::string, double> > getPeakFamilies(const std::vector<std::string> & elementList, \
                                              const double & energy) const;
 
+
+    /*!
+    Convenience function to simplify python use ...
+    */
+    const double & getAtomicMass(const std::string & name) const;
+
+    /*!
+    Convenience function to simplify python use ...
+    */
+    const int & getAtomicNumber(const std::string & name) const;
+
+
     /*!
     Convenience function to simplify python use ...
     */
     const std::map<std::string, double> & getBindingEnergies(const std::string & name) const;
 
+    /*!
+    Convenience function to simplify python use ...
+    */
+    double getDensity(const std::string & name) const;
+
+    /*!
+    Convenience function to simplify python use ...
+    */
+    std::string getLongName(const std::string & name) const;
+
+    /*!
+    Convenience function to simplify python use ...
+    */
+    int getColumn(const std::string & name) const;
+
+    /*!
+    Convenience function to simplify python use ...
+    */
+    int getRow(const std::string & name) const;
+
     /*!
     Return escape peak energy and rate per detected photon of given energy.
     Given a detector composition and an incident energy in keV, gives back a map of the form:
@@ -399,7 +441,17 @@ public:
                                         const double & alphaIn = 90.,\
                                         const double & thickness = 0.0) const;
 
-
+    /*!
+    Calculate the expected escape and stores it into cache.
+    The cache will be emptied if needed.
+    */
+    void updateEscapeCache(const std::map<std::string, double> & composition, \
+                                        const std::vector<double> & energy, \
+                                        const double & energyThreshold = 0.010, \
+                                        const double & intensityThreshold = 1.0e-7, \
+                                        const int & nThreshold = 4 , \
+                                        const double & alphaIn = 90.,\
+                                        const double & thickness = 0.0);
 
     /*!
     Optimization methods to keep the complete emission cascade following a single vacancy in a shell
@@ -409,6 +461,46 @@ public:
     int isElementCascadeCacheFilled(const std::string & elementName) const;
     void fillElementCascadeCache(const std::string & elementName);
     void emptyElementCascadeCache(const std::string & elementName);
+    void setEscapeCacheEnabled(const int & flag = 1){this->escapeCacheEnabled = flag;};
+    int isEscapeCacheEnabled() const {return this->escapeCacheEnabled;};
+
+    /*!
+    Optimization methods to keep the calculations at a set of energies in cache
+    Clear the calculation cache of given element and fill it at the selected energies
+    */
+    void fillCache(const std::string & elementName, const std::vector< double> & energy);
+
+    /*!
+    Enable or disable the use of the stored calculations (if any).
+    It does not clear the cache when disabling.
+    */
+    void setCacheEnabled(const std::string & elementName, const int & flag = 1);
+
+    /*!
+    Update the cache with those energy values not already present.
+    The existing values will be kept.
+    */
+    void updateCache(const std::string & elementName, const std::vector< double> & energy);
+
+    /*!
+    Clear the calculation cache
+    */
+    void clearCache(const std::string & elementName);
+
+    /*!
+    Clear the escape peak calculation cache
+    */
+    void clearEscapeCache(void);
+
+    /*!
+    Return 1 if the calculation cache is enabled
+    */
+    const int isCacheEnabled(const std::string & elementName) const;
+
+    /*!
+    Return the number of energies for which the calculations are stored
+    */
+    int getCacheSize(const std::string & elementName) const;
 
     /*!
     Utility to convert from string to double.
@@ -445,6 +537,23 @@ private:
     std::map<std::string, std::string> shellRadiativeTransitionsFile;
     std::map<std::string, std::string> shellNonradiativeTransitionsFile;
 
+    // A cache of escape peaks
+    std::map< double, std::map<std::string,std::map<std::string, double> > > detectorEscapeCache;
+    std::map< std::string, double> detectorCompositionUsedInCache;
+    double detectorEnergyThresholdUsedInCache;
+    double detectorIntensityThresholdUsedInCache;
+    int detectorNThresholdUsedInCache;
+    double detectorAlphaInUsedInCache;
+    double detectorThicknessUsedInCache;
+    int escapeCacheEnabled;
+    bool isEscapeCacheCompatible(\
+                                        const std::map<std::string, double> & composition,
+                                        const double & energyThreshold, \
+                                        const double & intensityThreshold, \
+                                        const int & nThreshold , \
+                                        const double & alphaIn , \
+                                        const double & thickness) const;
+
     struct sortVectorOfExcited {
         bool operator()(const std::pair<std::string, double> &left, const std::pair<std::string,int> &right) {
         return left.second < right.second;
