@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2016 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -30,6 +30,7 @@
 // TODO #include "fisx_version.h"
 #include "fisx_detector.h"
 #include "fisx_beam.h"
+#include "fisx_transmissiontable.h"
 
 namespace fisx
 {
@@ -63,6 +64,12 @@ public:
                         const std::vector<std::string> & comments);
     void setBeamFilters(const std::vector<Layer> & filters);
 
+    /*!
+    Set the list of user beam filters defined as transmission tables.
+    */
+    void setUserBeamFilters(const std::vector<TransmissionTable> & userBeamFilters);
+
+
     /*!
     Set the excitation geometry.
     For the time being, just the incident, outgoing angles and scattering angle to detector center.
@@ -97,6 +104,11 @@ public:
                         const std::vector<double> & thicknesses, \
                         const std::vector<std::string> & comments);
 
+    /*!
+    Set the list of user attenuators as transmission tables between sample and detector.
+    */
+    void setUserAttenuators(const std::vector<TransmissionTable> & userAttenuators);
+
     /*!
     Collimators are not implemented yet. The collimators are attenuators that take into account their distance to
     the sample, their diameter, thickness and density
@@ -125,8 +137,10 @@ public:
     */
    const Beam & getBeam() const;
    const std::vector<Layer> & getBeamFilters() const {return this->beamFilters;};
+   const std::vector<TransmissionTable> & getUserBeamFilters() const {return this->userBeamFilters;};
    const std::vector<Layer> & getSample() const {return this->sample;};
    const std::vector<Layer> & getAttenuators() const {return this->attenuators;};
+   const std::vector<TransmissionTable> & getUserAttenuators() const {return this->userAttenuators;};
    const Detector & getDetector() const {return this->detector;};
    const double & getAlphaIn() const {return this->alphaIn;};
    const double & getAlphaOut() const {return this->alphaOut;};
@@ -139,6 +153,8 @@ private:
     std::vector<Layer> beamFilters;
     std::vector<Layer> sample;          // just other layer with funny factor set to 1.0
     std::vector<Layer> attenuators;
+    std::vector<TransmissionTable> userBeamFilters;
+    std::vector<TransmissionTable> userAttenuators;
     int referenceLayer;
     double  alphaIn;
     double  alphaOut;
